﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.Reflection;
using System.IO;
using System.IO.IsolatedStorage;
using System.Xml.Serialization;

namespace DummyWP7App
{
    public class WeightUnitsModel : INotifyPropertyChanged
    {
        public static bool SaveWeightUnitsModel(WeightUnitsModel settings)
        {

            bool result = false;
            try
            {
                using (var store = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    if (!store.DirectoryExists("mystore"))
                    {
                        store.CreateDirectory("mystore");
                    }
                    using (var stream = new IsolatedStorageFileStream("mystore/weightunitsmodel.txt", FileMode.Create, FileAccess.Write, store))
                    {
                        var serializer = new XmlSerializer(typeof(WeightUnitsModel));
                        serializer.Serialize(stream, settings);
                        result = true;
                    }
                }
            }
            catch
            {
                throw new Exception("Failed to save to isolated storage!");
            }

            return result;
        }

        public static WeightUnitsModel GetWeightUnitsModel()
        {
            try
            {
                using (var store = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    if (store.DirectoryExists("mystore"))
                    {
                        using (var stream = new IsolatedStorageFileStream("mystore/weightunitsmodel.txt", FileMode.OpenOrCreate, FileAccess.Read, store))
                        using (var reader = new StreamReader(stream))
                        {
                            var serializer = new XmlSerializer(typeof(WeightUnitsModel));
                            return reader.EndOfStream ? new WeightUnitsModel() : (WeightUnitsModel)serializer.Deserialize(reader);
                        }
                    }
                    else
                    {
                        return new WeightUnitsModel();
                    }
                }
            }
            catch
            {
                throw new Exception("Failed to retrieve Weight Units Model from isolated storage!");
            }
        }

        public static decimal ConvertWeight(decimal value, string currentType, string wantedType)
        {
            WeightUnitsModel weight = new WeightUnitsModel();
            Type d = weight.GetType();
            PropertyInfo pCurrent = d.GetProperty(currentType);
            PropertyInfo pWanted = d.GetProperty(wantedType);
            pCurrent.SetValue(weight, value.ToString(), null);
            decimal returnMe = 0;
            var result = decimal.TryParse(pWanted.GetValue(weight, null).ToString(), out returnMe);
            return returnMe;
        }

        private bool isForFocusedTextboxes = false;

        public bool IsForFocusedTextboxes
        {
            get { return isForFocusedTextboxes; }
            set { isForFocusedTextboxes = value; }
        }

        private decimal _ounces = 16;

        public string Ounces
        {
            get { return _ounces.ToString("#0.##"); }
            set
            {
                decimal valParse = 0;
                bool passParse = decimal.TryParse(value.Trim(), out valParse);
                _ounces = valParse;
                this._pounds = _ounces / 16;
                this._grams = _pounds * (decimal)453.59237;
                this._kilograms = _grams / 1000;
                if (!IsForFocusedTextboxes)
                    NotifyPropertyChanged("Ounces");
                NotifyPropertyChanged("Pounds");
                NotifyPropertyChanged("Grams");
                NotifyPropertyChanged("Kilograms");

            }
        }
        private decimal _pounds = 1;

        public string Pounds
        {
            get { return _pounds.ToString("#0.##"); }
            set
            {

                decimal valParse = 0;
                bool passParse = decimal.TryParse(value.Trim(), out valParse);

                _pounds = valParse;
                this._ounces = _pounds * 16;
                this._kilograms = (decimal)0.45359237 * _pounds;
                this._grams = _kilograms * 1000;
                if (!IsForFocusedTextboxes)
                    NotifyPropertyChanged("Pounds");
                NotifyPropertyChanged("Ounces");
                NotifyPropertyChanged("Grams");
                NotifyPropertyChanged("Kilograms");
            }
        }

        private decimal _grams = (decimal)453.59237;

        public string Grams
        {
            get { return _grams.ToString("#0.##"); }
            set
            {
                decimal valParse = 0;
                bool passParse = decimal.TryParse(value.Trim(), out valParse);
                _grams = valParse;
                this._kilograms = _grams / 1000;
                this._ounces = _grams * (decimal)0.035273962;
                this._pounds = (decimal)2.2046226 * _kilograms;
                if (!IsForFocusedTextboxes)
                    NotifyPropertyChanged("Grams");
                NotifyPropertyChanged("Kilograms");
                NotifyPropertyChanged("Ounces");
                NotifyPropertyChanged("Pounds");

            }
        }
        private decimal _kilograms = (decimal)0.45359;

        public string Kilograms
        {
            get { return _kilograms.ToString("#0.##"); }
            set
            {

                decimal valParse = 0;
                bool passParse = decimal.TryParse(value.Trim(), out valParse);
                _kilograms = valParse;
                this._grams = _kilograms * 1000;
                this._ounces = (decimal)35.273962 * _kilograms;
                this._pounds = (decimal)2.2046226 * _kilograms;
                if (!IsForFocusedTextboxes)
                    NotifyPropertyChanged("Kilograms");
                NotifyPropertyChanged("Grams");
                NotifyPropertyChanged("Ounces");
                NotifyPropertyChanged("Pounds");
            }
        }



        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }
}
